/*
 * Software fr ATMega8 und kompatible
 * fr PWM-Board turi
 * Basissoftware ausgelegt fr:
 *    RC5-Fernbedienung (Port D2)
 *    3 PWM-Kanle mit Helligkeitsregelung fr 8-Bit PWM
 *    4 ADC-Kanal fr Potis RGB/HSB
 * mit Board erweiterbar bis auf 4 ADC-Kanle und 3 PWM
 * Software-Komponenten verschiedener Foren verwendet und teilweise adaptiert (s. Doku
 * der jeweiligen Files)
 *
 */


// Komponenten fr RC5 und ADC 
#include "RC5_v1.c"
#include "adc.c"

#include <avr/pgmspace.h>
#include <avr/eeprom.h> 
#include <util/delay.h>

//EPROM-Speicher
uint16_t ee_pwm1 EEMEM = 10; //Red
uint16_t ee_pwm2 EEMEM = 10; //Green
uint16_t ee_pwm3 EEMEM = 10; //Blue
uint16_t ee_pwm4 EEMEM = 10; //brightness
uint16_t ee_pwm5 EEMEM = 10; //sat
uint16_t ee_pwm6 EEMEM = 10; //hue
uint8_t ee_pwm7 EEMEM = 0;  //Programm
 
uint16_t bri_table_8[64]  PROGMEM = {0,0,1,1,1,2,2,2,2,2,2,3,3,3,3,4,4,4,5,5,6,
									6,7,8,8,9,10,11,12,13,14,15,17,18,20,22,24,
									26,28,31,34,37,40,44,48,52,57,62,68,74,81,89,
									97,106,116,126,138,150,164,179,196,214,234,255};


int main (void) {            
 
   	DDRB  = 0xff;             
   	PORTB = 0b00001111;            
   	PORTB = 0x07;            

   	//Werte des ADC
	uint16_t ADCval, ADCvalC1, ADCvalC2, ADCvalC3, ADCvalC0 ;		
	int ADCdiff=0;

	//RC5-Code, Adresse
	uint8_t code, addr;			


	//Variablen fr Farben etc.
	int16_t hue, sat, bri, red, blue, green, bri_val, red_val, green_val, blue_val;

	uint8_t key=0;		//Taste fr Farbwahl (1,2,3); Taste 5 zum Einleiten Speichern
	uint8_t iProg=0;	//Programmart (0/9,6)

	//0 - Standard RGB mit Helligkeit
	//6 - HSB mit Helligkeit
	//9 - RGB ohne Helligkeit


	//8-Bit-PWM initialisieren
	TCCR1A = 0xA1;			//  8 Bit Phase correct PWM 
    TCCR1B &= ~0x7;         // clear clk setting
    TCCR1B |= 2;            // Prescale 8 
	TCCR2 = (1<<COM21) | (0<<COM20) | (0<<WGM21) | (1<<WGM20) | (0<<CS22) | (1<<CS21) | (0<<CS20);

	//Fernbedinung aktivieren
	rc5_init (RC5_ALL);

	/* Interrupts zulassen */
	sei();


	// Kanal 0,1 ADC einmalig lesen, um Startwerte zu setzen
	ADCval = ReadChannel(3);ADCvalC3=ADCval; /* MUX-Bits auf 0b0000 -> Channel 3 */
	ADCval = ReadChannel(2);ADCvalC2=ADCval; /* MUX-Bits auf 0b0000 -> Channel 2 */
	ADCval = ReadChannel(1);ADCvalC1=ADCval; /* MUX-Bits auf 0b0000 -> Channel 1 */
	ADCval = ReadChannel(0);ADCvalC0=ADCval; /* MUX-Bits auf 0b0000 -> Channel 0 */

	//EEPROM lesen und Werte setzen
	red_val=eeprom_read_word(&ee_pwm1);sat=red_val;
	green_val=eeprom_read_word(&ee_pwm2);hue=green_val;
	blue_val=eeprom_read_word(&ee_pwm3);
	bri_val=eeprom_read_word(&ee_pwm4);
	sat=eeprom_read_word(&ee_pwm5);
	hue=eeprom_read_word(&ee_pwm6);
	iProg=eeprom_read_byte(&ee_pwm7);
	bri=pgm_read_word(bri_table_8+bri_val);

	//Hauptschleife
   	while(1) {
			                
		   if (-1 == rc5.flip) //keine FB - Potiabfrage
		   {
		   		//keine FB-Taste gedrckt: Kanal 0-3 ADC lesen
				ADCval = ReadChannel(3); 
				//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
				if(ADCvalC3<2) ADCdiff=ADCvalC3; else ADCdiff=2;
				if ((ADCval > ADCvalC3+2) || (ADCval < ADCvalC3-ADCdiff))
		 		{ 
					//Helligkeit gesamt ber Tabelle logarithmisch
					ADCvalC3=ADCval;
					bri_val=ADCvalC3/16;  //0..63
				}				
				ADCval = ReadChannel(2); 
				//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
				if(ADCvalC2<2) ADCdiff=ADCvalC2; else ADCdiff=2;
				if ((ADCval > ADCvalC2+2) || (ADCval < ADCvalC2-ADCdiff))
				{
					//ADC-Wert speichern
					ADCvalC2=ADCval;
					blue_val=ADCvalC2/4; //0..255
				}
				ADCval = ReadChannel(1); /* MUX-Bits auf 0b0000 -> Channel 1 */
				//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
				if(ADCvalC1<2) ADCdiff=ADCvalC1; else ADCdiff=2;
				if ((ADCval > ADCvalC1+2) || (ADCval < ADCvalC1-ADCdiff))
				{
					//ADC-Wert speichern
					ADCvalC1=ADCval;
					green_val=ADCvalC1/4; //0..255
					hue = ADCvalC1*3/4  ; //0..765
				}
				ADCval = ReadChannel(0); /* MUX-Bits auf 0b0000 -> Channel 0 */
				//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
				if(ADCvalC0<2) ADCdiff=ADCvalC0; else ADCdiff=2;
				if ((ADCval > ADCvalC0+2) || (ADCval < ADCvalC0-ADCdiff))
				{
					//ADC-Wert speichern
					ADCvalC0=ADCval;
					red_val=ADCvalC0/4; //0..255
					sat = ADCvalC0/8;  //0..128
				}
		   }  //keine FB - Potiabfrage
		   else
		   {	//FB verarbeiten
		      	/* Ja, dann rc5.code merken und evtl. rc5.addr */
		      	code = rc5.code;
		      	addr = rc5.addr;
		      	/* und auf naechstes Zeichen warten */
		      	rc5.flip = -1;
			  if (addr==0)		//TV-Modus
			  {
				  switch (code)
				  {
					case 12: { 	red_val=0;green_val=0;blue_val=0;bri=0;
								break;}    // Aus
		    		case 43: { 	red_val=0;green_val=64;blue_val=64;
								break;}    // Play
		    		case 44: { 	red_val=128;green_val=0;blue_val=128;
								break;}    // Stop
		    		case 53: { 	red_val=200;green_val=200;blue_val=0;
								break;}    // Pause
		    		case 13: { 	red_val=255;green_val=255;blue_val=255;
								break;}    // Mute
		    		case 60: { 	red_val=255;green_val=0;blue_val=0;
								break;}    // Record
		    		case 46: { 	red_val=0;green_val=255;blue_val=0;
								break;}    // back
		    		case 42: { 	if (key==5)
								{	//Speichern der akt. Werte eingeleitet
									eeprom_write_word(&ee_pwm1,red_val ); //rot speichern
									eeprom_write_word(&ee_pwm2,green_val ); //grn speichern
									eeprom_write_word(&ee_pwm3,blue_val ); //blau speichern
									eeprom_write_word(&ee_pwm4,bri_val ); //akt. Helligkeit speichern
									eeprom_write_word(&ee_pwm5,sat ); //sat speichern
									eeprom_write_word(&ee_pwm6,hue ); //hue speichern
									eeprom_write_byte(&ee_pwm7,iProg ); //Programm speichern
									//eeprom_busy_wait();
									//Besttigung kurz blinken
									OCR1A = 20;_delay_ms(100);
									OCR1A = 160;_delay_ms(100);
									OCR1A = 20;_delay_ms(100);
									OCR1A = 160;_delay_ms(100);
									OCR1A = red_val;_delay_ms(20);
									key=0;
								}
								else
								{	red_val=0;green_val=0;blue_val=255;}
									break;
								}    // forward

		    		case 16:// Vol + in Abhngigkeit Zifferntaste
					 		{ 
								if (key==1)
								{
									red_val+=2;
									sat+=2;
									if (sat > 128) sat = 128;
									if (red_val > 255) red_val = 255;
								}
								if (key==2)
								{
									green_val+=2;
									hue+=2;
									if (hue > 764) hue = 764;
									if (green_val > 255) green_val = 255;
								}
								if (key==3)
								{
									blue_val+=2;
									if (blue_val > 255) blue_val = 255;
								}
								break;
							}    
		    		case 17:// Vol - in Abhngigkeit Zifferntaste
							 { 
								if (key==1)
								{
									red_val-=2;
									sat-=2;
									if (sat < 0) sat = 0;
									if (red_val < 0) red_val = 0;
								}
								if (key==2)
								{
									green_val-=2;
									hue-=2;
									if (green_val < 0) green_val = 0;
									if (hue < 0) hue = 0;
								}
								if (key==3)
								{
									blue_val-=2;
									if (blue_val < 0) blue_val = 0;
								}

								break;}    
		    		case 32: { // Ch + 
									bri_val+=2;
									if (bri_val > 63) bri_val = 63;
								break;
							}    
		    		case 33: { // Ch - 
									bri_val-=2;
									if (bri_val < 0) bri_val = 0;
								break;}    

					case 1:	{	key=1; break;}		//Farbe 1
					case 2:	{	key=2; break;}		//Farbe 2
					case 3:	{	key=3; break;}		//Farbe 3
					case 5:	{	key=5; break;}		//Speichern einleiten
					case 6:	{	iProg=6; break;}	//Prog HSB
					case 9: {	iProg=9; break;}	//Prog RGB / bri
					case 0: {	iProg=0; break;}	//Prog RGB
		    		default: ;    

				  } //switch (code)

			  } //addr==0
      

		   } //FB-Verarbeitung


			//Helligkeit ber Lookup-Tabelle berechnen
			bri=pgm_read_word(bri_table_8+bri_val);  //0..255

			//HSB-Werte aus Potis/FB interpretieren (Taste 6 an FB gedrckt)
			if (iProg==6)
			{		   
				if (hue > 764) hue = 0;

				if (hue < 255) {
				   red_val = (10880 - sat * (hue - 170)) >> 7;
				   green_val = (10880 - sat * (85 - hue)) >> 7;
				   blue_val = (10880 - sat * 85) >> 7;
				}
				else if (hue < 510) {
				   red_val = (10880 - sat * 85) >> 7;
				   green_val = (10880 - sat * (hue - 425)) >> 7;
				   blue_val = (10880 - sat * (340 - hue)) >> 7;
				}
				else {
				   red_val = (10880 - sat * (595 - hue)) >> 7;
				   green_val = (10880 - sat * 85) >> 7;
				   blue_val = (10880 - sat * (hue - 680)) >> 7;
				}
				//im HSB-Programm geht Helligkeit bis 255 (*4)
				red = ((bri + 1) * red_val) >> 8;
				green = ((bri + 1) * green_val) >> 8;
				blue = ((bri + 1) * blue_val) >> 8;		
				OCR1A = red;
				OCR1B = green;
				OCR2 = blue;		
			}
			else if (iProg==0)	//RGB-Werte + Helligkeit verarbeiten Prog 0
			{
				//Helligkeit Multiplizieren fr Prog 0
				red = ((bri + 1) * red_val) >> 8;
				green = ((bri + 1) * green_val) >> 8;
				blue = ((bri + 1) * blue_val) >> 8;		   
				OCR1A = red;
				OCR1B = green;
				OCR2 = blue;		
			}
			else if (iProg==9)	//RGB ohne Helligkeitsregelung
			{
				//direkte Ausgabe der Werte
				OCR1A = red_val;
				OCR1B = green_val;
				OCR2 = blue_val;		
			}
	  	//else Fernbedienung


   }    //while                     
 
   return 0;                 
}
